@@ -48,8 +48,8 @@ except ImportError:
 __author__ = "Marius Gedminas (marius@gedmin.as)"
 __copyright__ = "Copyright (c) 2008-2015 Marius Gedminas and contributors"
 __license__ = "MIT"
-__version__ = "2.0.0"
-__date__ = "2015-04-14"
+__version__ = "2.0.1"
+__date__ = "2015-07-28"
 
 
 try:
@@ -89,10 +89,13 @@ def count(typename, objects=None):
     """
     if objects is None:
         objects = gc.get_objects()
-    if '.' in typename:
-        return sum(1 for o in objects if _long_typename(o) == typename)
-    else:
-        return sum(1 for o in objects if _short_typename(o) == typename)
+    try:
+        if '.' in typename:
+            return sum(1 for o in objects if _long_typename(o) == typename)
+        else:
+            return sum(1 for o in objects if _short_typename(o) == typename)
+    finally:
+        del objects  # clear cyclic references to frame
 
 
 def typestats(objects=None, shortnames=True):
@@ -121,15 +124,18 @@ def typestats(objects=None, shortnames=True):
     """
     if objects is None:
         objects = gc.get_objects()
-    if shortnames:
-        typename = _short_typename
-    else:
-        typename = _long_typename
-    stats = {}
-    for o in objects:
-        n = typename(o)
-        stats[n] = stats.get(n, 0) + 1
-    return stats
+    try:
+        if shortnames:
+            typename = _short_typename
+        else:
+            typename = _long_typename
+        stats = {}
+        for o in objects:
+            n = typename(o)
+            stats[n] = stats.get(n, 0) + 1
+        return stats
+    finally:
+        del objects  # clear cyclic references to frame
 
 
 def most_common_types(limit=10, objects=None, shortnames=True):
@@ -256,7 +262,7 @@ def get_leaking_objects(objects=None):
         # this then is our set of objects without referrers
         return [i for i in objects if id(i) in ids]
     finally:
-        objects = i = None  # clear cyclic references to frame
+        del objects, i  # clear cyclic references to frame
 
 
 def by_type(typename, objects=None):
@@ -279,10 +285,13 @@ def by_type(typename, objects=None):
     """
     if objects is None:
         objects = gc.get_objects()
-    if '.' in typename:
-        return [o for o in objects if _long_typename(o) == typename]
-    else:
-        return [o for o in objects if _short_typename(o) == typename]
+    try:
+        if '.' in typename:
+            return [o for o in objects if _long_typename(o) == typename]
+        else:
+            return [o for o in objects if _short_typename(o) == typename]
+    finally:
+        del objects  # clear cyclic references to frame
 
 
 def at(addr):
@@ -414,7 +423,7 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     .. versionchanged:: 1.8
        New parameter: ``shortnames``.
 
-    .. versionchanged:: 1.9
+    .. versionchanged:: 2.0
        New parameter: ``output``.
 
     """
@@ -487,7 +496,7 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     .. versionchanged:: 1.8
        New parameter: ``shortnames``.
 
-    .. versionchanged:: 1.9
+    .. versionchanged:: 2.0
        New parameter: ``output``.
     """
     _show_graph(objs, max_depth=max_depth, extra_ignore=extra_ignore,
@@ -524,7 +533,7 @@ def show_chain(*chains, **kw):
     .. versionchanged:: 1.7
        New parameter: ``backrefs``.
 
-    .. versionchanged:: 1.9
+    .. versionchanged:: 2.0
        New parameter: ``output``.
 
     """
@@ -548,12 +557,12 @@ def is_proper_module(obj):
 
     That is, if ``obj`` is a module that is in ``sys.modules``.
 
-    >>> import imp
+    >>> import types
     >>> is_proper_module([])
     False
-    >>> is_proper_module(imp)
+    >>> is_proper_module(types)
     True
-    >>> is_proper_module(imp.new_module('foo'))
+    >>> is_proper_module(types.ModuleType('foo'))
     False
 
     .. versionadded:: 1.8
@@ -727,8 +736,21 @@ def _show_graph(objs, edge_func, swap_source_target,
     # the file.
     f.close()
     print("Graph written to %s (%d nodes)" % (dot_filename, nodes))
-    if filename and filename.endswith('.dot'):
-        # nothing else to do, the user asked for a .dot file
+    _present_graph(dot_filename, filename)
+
+
+def _present_graph(dot_filename, filename=None):
+    """Present a .dot file to the user in the requested fashion.
+
+    If ``filename`` is provided, runs ``dot`` to convert the .dot file
+    into the desired format, determined by the filename extension.
+
+    If ``filename`` is not provided, tries to launch ``xdot``, a
+    graphical .dot file viewer.  If ``xdot`` is not present on the system,
+    converts the graph to a PNG.
+    """
+    if filename == dot_filename:
+        # nothing to do, the user asked for a .dot file and got it
         return
     if not filename and _program_in_path('xdot'):
         print("Spawning graph viewer (xdot)")
@@ -738,23 +760,21 @@ def _show_graph(objs, edge_func, swap_source_target,
             print("Graph viewer (xdot) not found, generating a png instead")
             filename = dot_filename[:-4] + '.png'
         stem, ext = os.path.splitext(filename)
-        f = open(filename, 'wb')
-        cmd = ['dot', ('-T' + ext[1:]), dot_filename]
-        dot = subprocess.Popen(cmd, stdout=f, close_fds=False)
+        cmd = ['dot', '-T' + ext[1:], '-o' + filename, dot_filename]
+        dot = subprocess.Popen(cmd, close_fds=False)
         dot.wait()
         if dot.returncode != 0:
             # XXX: shouldn't this go to stderr or a log?
             print('dot failed (exit code %d) while executing "%s"'
                   % (dot.returncode, ' '.join(cmd)))
-        f.close()
-        print("Image generated as %s" % filename)
+        else:
+            print("Image generated as %s" % filename)
     else:
-        if filename:
+        if not filename:
             print("Graph viewer (xdot) and image renderer (dot) not found,"
                   " not doing anything else")
         else:
-            print("Unrecognized file type (%s), not doing anything else"
-                  % filename)
+            print("Image renderer (dot) not found, not doing anything else")
 
 
 def _obj_node_id(obj):
@@ -894,6 +914,7 @@ _is_identifier = re.compile('[a-zA-Z_][a-zA-Z_0-9]*$').match
 
 
 def _program_in_path(program):
+    # XXX: Consider using distutils.spawn.find_executable or shutil.which
     path = os.environ.get("PATH", os.defpath).split(os.pathsep)
     path = [os.path.join(dir, program) for dir in path]
     path = [True for file in path
