@@ -67,8 +67,45 @@ class GarbageCollectedMixin(object):
     """A mixin for test cases that garbage collects before running."""
 
     def setUp(self):
+        super(GarbageCollectedMixin, self).setUp()
         gc.collect()
 
+    def tearDown(self):
+        super(GarbageCollectedMixin, self).tearDown()
+        gc.enable()
+
+
+class CaptureMixin(object):
+    """A mixing that captures sys.stdout"""
+
+    def setUp(self):
+        super(CaptureMixin, self).setUp()
+        self.real_stdout = sys.stdout
+        sys.stdout = StringIO()
+
+    def tearDown(self):
+        sys.stdout = self.real_stdout
+        super(CaptureMixin, self).tearDown()
+
+    def assertOutput(self, output):
+        self.assertEqual(sys.stdout.getvalue(),
+                         textwrap.dedent(output.lstrip('\n')))
+
+
+class TemporaryDirectoryMixin(object):
+    """A mixin that sets up a temporary directory"""
+
+    def setUp(self):
+        super(TemporaryDirectoryMixin, self).setUp()
+        self.prevdir = os.getcwd()
+        self.tmpdir = tempfile.mkdtemp(prefix='test-objgraph-')
+        os.chdir(self.tmpdir)
+
+    def tearDown(self):
+        os.chdir(self.prevdir)
+        shutil.rmtree(self.tmpdir)
+        super(TemporaryDirectoryMixin, self).tearDown()
+
 
 # Unit tests
 
@@ -196,6 +233,15 @@ class CountTest(GarbageCollectedMixin, unittest.TestCase):
         self.assertEqual(2, objgraph.count('MyClass'))
         self.assertEqual(1, objgraph.count('mymodule.MyClass'))
 
+    def test_no_new_reference_cycles(self):
+        # Similar to https://github.com/mgedmin/objgraph/pull/22 but for
+        # count()
+        gc.disable()
+        x = type('MyClass', (), {})()
+        self.assertEqual(len(gc.get_referrers(x)), 1)
+        objgraph.count('MyClass')
+        self.assertEqual(len(gc.get_referrers(x)), 1)
+
 
 class TypestatsTest(GarbageCollectedMixin, unittest.TestCase):
     """Tests for the typestats function."""
@@ -205,6 +251,15 @@ class TypestatsTest(GarbageCollectedMixin, unittest.TestCase):
         stats = objgraph.typestats(shortnames=False)
         self.assertEqual(1, stats['mymodule.MyClass'])
 
+    def test_no_new_reference_cycles(self):
+        # Similar to https://github.com/mgedmin/objgraph/pull/22 but for
+        # typestats()
+        gc.disable()
+        x = type('MyClass', (), {})()
+        self.assertEqual(len(gc.get_referrers(x)), 1)
+        objgraph.typestats()
+        self.assertEqual(len(gc.get_referrers(x)), 1)
+
 
 class ByTypeTest(GarbageCollectedMixin, unittest.TestCase):
     """Tests for the by_test function."""
@@ -213,6 +268,19 @@ class ByTypeTest(GarbageCollectedMixin, unittest.TestCase):
         x = type('MyClass', (), {'__module__': 'mymodule'})()
         self.assertEqual([x], objgraph.by_type('mymodule.MyClass'))
 
+    def test_new_garbage(self):
+        # Regression test for https://github.com/mgedmin/objgraph/pull/22
+        gc.disable()
+        x = type('MyClass', (), {})()
+        res = objgraph.by_type('MyClass')
+        self.assertEqual(res, [x])
+        # referrers we expect:
+        # 1. this stack frame
+        # 2. the `res` list
+        # referrers we don't want:
+        # the ``objects`` list in the now-dead stack frame of objgraph.by_type
+        self.assertEqual(len(gc.get_referrers(res[0])), 2)
+
 
 class StringRepresentationTest(GarbageCollectedMixin,
                                CompatibilityMixin,
@@ -269,6 +337,97 @@ class StringRepresentationTest(GarbageCollectedMixin,
             ' [label="mymodule\.MyClass\\n<mymodule\.MyClass object at .*"]')
 
 
+class StubSubprocess(object):
+
+    should_fail = False
+
+    def Popen(self, args, close_fds=False):
+        return StubPopen(args, close_fds=close_fds,
+                         should_fail=self.should_fail)
+
+
+class StubPopen(object):
+
+    def __init__(self, args, close_fds=False, should_fail=False):
+        print("subprocess.Popen(%s)" % repr(args))
+        self.args = args
+        self.should_fail = should_fail
+
+    def wait(self):
+        self.returncode = int(self.should_fail)
+
+
+class PresentGraphTest(CaptureMixin, TemporaryDirectoryMixin,
+                       unittest.TestCase):
+
+    def setUp(self):
+        super(PresentGraphTest, self).setUp()
+        self.orig_subprocess = objgraph.subprocess
+        self.orig_program_in_path = objgraph._program_in_path
+        objgraph.subprocess = StubSubprocess()
+        self.programsInPath([])
+
+    def tearDown(self):
+        objgraph._program_in_path = self.orig_program_in_path
+        objgraph.subprocess = self.orig_subprocess
+        super(PresentGraphTest, self).tearDown()
+
+    def programsInPath(self, programs):
+        objgraph._program_in_path = set(programs).__contains__
+
+    def test_present_dot(self):
+        objgraph._present_graph('foo.dot', 'foo.dot')
+        self.assertOutput("")
+
+    def test_present_png(self):
+        self.programsInPath(['dot'])
+        objgraph._present_graph('foo.dot', 'bar.png')
+        self.assertOutput("""
+            subprocess.Popen(['dot', '-Tpng', '-obar.png', 'foo.dot'])
+            Image generated as bar.png
+        """)
+
+    def test_present_png_failure(self):
+        self.programsInPath(['dot'])
+        objgraph.subprocess.should_fail = True
+        objgraph._present_graph('f.dot', 'b.png')
+        self.assertOutput("""
+            subprocess.Popen(['dot', '-Tpng', '-ob.png', 'f.dot'])
+            dot failed (exit code 1) while executing "dot -Tpng -ob.png f.dot"
+        """)
+
+    def test_present_png_no_dot(self):
+        self.programsInPath([])
+        objgraph._present_graph('foo.dot', 'bar.png')
+        self.assertOutput("""
+            Image renderer (dot) not found, not doing anything else
+        """)
+        self.assertFalse(os.path.exists('bar.png'))
+
+    def test_present_xdot(self):
+        self.programsInPath(['xdot'])
+        objgraph._present_graph('foo.dot')
+        self.assertOutput("""
+            Spawning graph viewer (xdot)
+            subprocess.Popen(['xdot', 'foo.dot'])
+        """)
+
+    def test_present_no_xdot(self):
+        self.programsInPath(['dot'])
+        objgraph._present_graph('foo.dot')
+        self.assertOutput("""
+            Graph viewer (xdot) not found, generating a png instead
+            subprocess.Popen(['dot', '-Tpng', '-ofoo.png', 'foo.dot'])
+            Image generated as foo.png
+        """)
+
+    def test_present_no_xdot_and_no_not(self):
+        self.programsInPath([])
+        objgraph._present_graph('foo.dot')
+        self.assertOutput("Graph viewer (xdot) and image renderer (dot)"
+                          " not found, not doing anything else\n")
+
+
 # Doctests
 
 
